const BbPromise = require('bluebird')
const ora = require('ora')
const chalk = require('chalk')

const clipboardy = require('clipboardy')

const getConfig = require('../utils/getConfig')
const fs = require('../utils/fs/index')
const regions = require('../utils/aws/regions')

const getOrCreateRole = require('../utils/aws/getOrCreateRole')
const createOrUpdateLambda = require('../utils/aws/createOrUpdateLambda')
const getOrCreateRestApi = require('../utils/aws/getOrCreateRestApi')

const { pack } = fs

module.exports = async () => {
  const spinner = ora({ text: 'Syncing', color: 'yellow' }).start()
  try {
    const config = await getConfig()
    const res = await BbPromise.all([
      getOrCreateRole(`${config.name}-lambda`, 'lambda.amazonaws.com', config.region),
      getOrCreateRole(`${config.name}-apig`, 'apigateway.amazonaws.com', config.region),
      pack()
    ])
    const lambdaRoleArn = res[0]
    const apigRoleArn = res[1]
    const pkgObj = res[2]
    const pkg = pkgObj.file
    spinner.text = `Syncing (${pkgObj.size})`
    const lambdaArn = await createOrUpdateLambda({ ...config, lambdaRoleArn, pkg })
    const restApiId = await getOrCreateRestApi({ ...config, apigRoleArn, lambdaArn })
    const regionId = Object.keys(regions).find((id) => regions[id] === config.region)
    const SID = `${regionId}-${restApiId}`
    await clipboardy.write(SID)
    spinner.succeed(`${chalk.bold('SID:')} ${SID} ${chalk.gray('(copied)')}`)
  } catch (e) {
    spinner.fail('Sync failed')
    console.log('')
    console.log(e)
  }
}
