const AWS = require('aws-sdk')
const R = require('ramda')
const swaggerDefinition = require('./swaggerDefinition.json')

const getSwaggerDefinition = ({ name, lambdaArn, apigRoleArn, region }) => {
  const uri = `arn:aws:apigateway:${region}:lambda:path/2015-03-31/functions/${lambdaArn}/invocations`

  swaggerDefinition.info.title = name
  swaggerDefinition.info.version = new Date().toISOString()
  swaggerDefinition.paths['/s'].post['x-amazon-apigateway-integration'].credentials = apigRoleArn
  swaggerDefinition.paths['/s'].post['x-amazon-apigateway-integration'].uri = uri

  return swaggerDefinition
}

const getRestApi = async (config) => {
  const APIGateway = new AWS.APIGateway({region: config.region})
  const resApis = await APIGateway.getRestApis({}).promise()
  const restApi = R.find(R.propEq('name', config.name))(resApis.items)
  return (restApi ? restApi.id : null)
}

const createRestApi = async (config) => {
  const APIGateway = new AWS.APIGateway({region: config.region})
  const swagger = getSwaggerDefinition(config)
  const json = JSON.stringify(swagger)

  const res = await APIGateway.importRestApi({
    body: Buffer.from(json, 'utf8')
  }).promise()

  await APIGateway.createDeployment({ restApiId: res.id, stageName: 'dev' }).promise()
  return res.id
}

module.exports = async (config) => {
  let endpoint = await getRestApi(config)
  if (!endpoint) {
    endpoint = await createRestApi(config)
  }
  return endpoint
}
