const path = require('path')
const archiver = require('archiver')
const os = require('os')
const fse = require('./fse')
const filesize = require('filesize')

module.exports = async () => {
  const outputFileName = `${String(Date.now())}.zip`
  const outputFilePath = path.join(os.tmpdir(), outputFileName)
  const pkg = {
    path: outputFilePath
  }
  return new Promise((resolve, reject) => { // eslint-disable-line

    const output = fse.createWriteStream(outputFilePath)
    const archive = archiver('zip', {
      zlib: {level: 9}
    })

    archive.on('error', (err) => reject(err))
    output.on('close', () => {
      const stats = fse.statSync(outputFilePath)
      pkg.size = filesize(stats.size)
      pkg.file = fse.readFileSync(outputFilePath)
      return resolve(pkg)
    })

    archive.pipe(output)
    const handlerTemplateFilePath = path.join(__dirname, '..', 'aws', 'handlerTemplate.js')
    archive.append(fse.createReadStream(handlerTemplateFilePath), {name: 'handler.js'})
    archive.glob('**/*', {
      cwd: process.cwd(),
      ignore: 'node_modules/aws-sdk/**' // todo it doesn't ignore aws-sdk dependencies. Eventually use webpack or rollup
    }, {})
    archive.finalize()
  })
}
